#!/usr/bin/python2.4
#
# Copyright 2010 Google Inc.
#
# This code is licensed under the same terms as WebM:
#  Software License Agreement:  http://www.webmproject.org/license/software/
#  Additional IP Rights Grant:  http://www.webmproject.org/license/additional/
# -----------------------------------------------------------------------------

"""Print file size and SSIM for webp and jpeg.

To run this script you need to provide correct paths for cjpeg, djpeg, cwebp
dwebp, convert and ssim binaries. Replace the corresponding variables below and
then run the following command.

$ ./equal-ssim.py /path/to/your/images/*.png

It will write the file size and SSIM information in the output file
"equal_ssim.txt". To set a different quality for the jpeg images change the
variable jpg_quality below.

- cjpeg and djpeg are available in libjpeg library (http://www.ijg.org/)
- cwebp and dwebp are available in libwebp library
  (http://code.google.com/speed/webp/)
- convert tool is available in ImageMagick (http://www.imagemagick.org/)
- SSIM implementation is available at http://mehdi.rabah.free.fr/SSIM/
  SSIM program should output a single float value for this script to be able
  to parse.
"""

import os
import sys
import time

# path of conversion tools
ssim = "/path/to/ssim"
cjpeg = "/path/to/cjpeg -optimize"
djpeg = "/path/to/djpeg"
cwebp = "/path/to/cwebp"
dwebp = "/path/to/dwebp"
convert = "/path/to/convert"

tmpdir = "/tmp/"
outfile = "equal_ssim.txt"
jpg_quality = 50


def GetWebPSSIM(png, q, webp_file, webp_png):
  # original png -> webp
  cmd = "%s -quiet -q %d %s -o %s" % (cwebp, q, png, webp_file)
  os.system(cmd)
  # webp => png
  cmd = "%s %s -o %s" % (dwebp, webp_file, webp_png)
  os.system(cmd)

  cmd = "%s %s %s" % (ssim, png, webp_png)
  proc = os.popen(cmd, "r")
  return float(proc.readline())


def main(argv):

  start_time = time.time()

  orig_size = 0
  jpg_size = 0
  webp_size = 0
  jpg_ssim = 0.0
  webp_ssim = 0.0
  total = 0

  for png in argv:
    if not os.path.exists(png) or not png.endswith(".png"):
      return
    # Define various temporary files
    filename = os.path.basename(png)
    basename = tmpdir + os.path.splitext(filename)[0]
    ppm_file = basename + ".ppm"
    jpg_file = basename + ".jpg"
    jpg_ppm = basename + "_jpg.ppm"
    jpg_png = basename + "_jpg.png"
    webp_file = basename + ".webp"
    webp_png = basename + "_webp.png"

    # original png => ppm
    cmd = "%s %s %s" % (convert, png, ppm_file)
    os.system(cmd)
    # ppm => jpg
    cmd = "%s -quality %d -outfile %s %s" % (cjpeg, jpg_quality, jpg_file,
                                             ppm_file)
    os.system(cmd)
    # jpg => ppm
    cmd = "%s -outfile %s %s" % (djpeg, jpg_ppm, jpg_file)
    os.system(cmd)
    # ppm => png
    cmd = "%s %s %s" % (convert, jpg_ppm, jpg_png)
    os.system(cmd)

    # jpeg ssim
    cmd = "%s %s %s" % (ssim, png, jpg_png)
    proc = os.popen(cmd, "r")
    j_ssim = float(proc.readline())

    # get a WebP image that macthes the SSIM as closely as possible
    q = 95
    curr_ssim = GetWebPSSIM(png, q, webp_file, webp_png)
    w_size = os.path.getsize(webp_file)
    w_ssim = curr_ssim

    delta_q = int(1000 * (j_ssim - curr_ssim))
    if q + delta_q > 100:
      delta_q = 100 - q
    elif q + delta_q < 0:
      delta_q = 1 - q

    test_q = q + delta_q
    while delta_q != 0:
      accept = False
      if test_q > 0 and test_q <= 100:
        curr_ssim = GetWebPSSIM(png, test_q, webp_file, webp_png)
        # accept if curr_ssim is in the same side of j_ssim.
        accept = (curr_ssim < j_ssim) ^ (delta_q < 0)

      if accept:
        q = test_q
        w_ssim = curr_ssim
        w_size = os.path.getsize(webp_file)
      else:
        delta_q = int(float(delta_q) / 2)

      test_q = q + delta_q

    # update the stats
    orig_size += os.path.getsize(png)
    jpg_size += os.path.getsize(jpg_file)
    webp_size += w_size
    jpg_ssim += j_ssim
    webp_ssim += w_ssim
    total += 1

    # delete the tmp files
    os.remove(ppm_file)
    os.remove(jpg_file)
    os.remove(jpg_ppm)
    os.remove(jpg_png)
    os.remove(webp_file)
    os.remove(webp_png)

  # output to a file
  out = ("Total_size = %d jpeg_size = %d jpeg_ssim = %f webp_size = %d "
         "webp_ssim = %f webp/jpeg =  %0.2fx\n") % (orig_size, jpg_size,
                                                    jpg_ssim / total,
                                                    webp_size,
                                                    webp_ssim / total,
                                                    float(webp_size) / jpg_size)

  fout = open(outfile, "w")
  fout.write(out)
  fout.close()

  print "Run time %d sec" % (time.time() - start_time)

if __name__ == "__main__":
  main(sys.argv[1:])
